/// <reference types="node" />
import { EventEmitter } from 'events';
import System from 'corpjs-system';
import Confabulous = require('confabulous');
export declare class Confabulous extends EventEmitter {
    static loaders: Config.LoaderRegistry;
    static processors: Config.ProcessorRegistry;
    add(cb: Config.AddCallback): this;
    end(cb: Config.EndCallback): this;
}
export declare class Config extends Confabulous {
    constructor();
    start(deps: System.ResourceDescriptor, restart: System.ComponentRestartFunction, stop: System.ComponentStopFunction): Promise<any>;
}
export declare namespace Config {
    const loaders: LoaderRegistry;
    const processors: ProcessorRegistry;
    type LoaderCallback = (err: Error, path: string) => EventEmitter;
    type LoaderFunction = (confabulous: Confabulous, cb: LoaderCallback) => void;
    type Processor = (...args: any[]) => void;
    type EndCallback = (err: Error, config: any) => void;
    type AddCallback = (config: any) => LoaderFunction;
    interface LoaderOptions {
        path?: string;
        mandatory?: boolean;
        watch?: boolean;
    }
    interface LoaderRegistry {
        require(options: LoaderOptions, processors?: Processor[]): LoaderFunction;
        file(options: LoaderOptions, processors?: Processor[]): LoaderFunction;
        env(processors?: Processor[]): LoaderFunction;
        args(processors?: Processor[]): LoaderFunction;
    }
    type ProcessorCallback = (config: any, cb: Function) => void;
    type ProcessorCallbackDecrypt = (encrypted: any, cb: Function) => void;
    type ProcessorFunctionOptions = (options: ProcessorOptions) => ProcessorCallback;
    type ProcessorFunctionNoOptions = () => ProcessorCallback;
    type ProcessorFunctionDecrypt = (encrypted: string, cb: Function) => ProcessorCallbackDecrypt;
    type ProcessorFunctionJson = (text: string, cb: Function) => void;
    interface ProcessorOptions {
        key?: string;
        algorithm?: string;
        password?: string;
        contentEncoding?: string;
        inputEncoding?: string;
    }
    interface ProcessorRegistry {
        mount(options: ProcessorOptions): ProcessorFunctionOptions;
        json(): ProcessorFunctionJson;
        decrypt(options: ProcessorOptions): ProcessorFunctionDecrypt;
        envToProp(): ProcessorCallback;
        flatten(): ProcessorFunctionNoOptions;
        unflatten(): ProcessorFunctionNoOptions;
    }
}
export default Config;
export declare const loaders: Config.LoaderRegistry;
export declare const processors: Config.ProcessorRegistry;
