import { EventEmitter } from 'events'
import System from 'corpjs-system'
import Confabulous = require('confabulous')

export declare class Confabulous extends EventEmitter {
  static loaders: Config.LoaderRegistry
  static processors: Config.ProcessorRegistry
  public add(cb: Config.AddCallback): this
  public end(cb: Config.EndCallback): this
}

export class Config extends Confabulous {

  constructor() {
    super()
    this.start = this.start.bind(this)
  }

  public start(deps: System.ResourceDescriptor, restart: System.ComponentRestartFunction, stop: System.ComponentStopFunction): Promise<any> {
    
    this.on('reloaded', () => {
      this.removeAllListeners()
      restart()
    })

    this.on('reload_error', stop)

    return new Promise<any>((resolve, reject) => {
      this.end((err, config) => {
        if (err) return reject(err)
        resolve(config)
      })
    })
  }

}

export namespace Config {

  export const loaders: LoaderRegistry = Confabulous.loaders
  export const processors: ProcessorRegistry = Confabulous.processors

  export type LoaderCallback = (err: Error, path: string) => EventEmitter
  export type LoaderFunction = (confabulous: Confabulous, cb: LoaderCallback) => void
  export type Processor = (...args: any[]) => void
  export type EndCallback = (err: Error, config: any) => void
  export type AddCallback = (config: any) => LoaderFunction

  export interface LoaderOptions {
    path?: string
    mandatory?: boolean
    watch?: boolean
  }

  export interface LoaderRegistry {
    require(options: LoaderOptions, processors?: Processor[]): LoaderFunction
    file(options: LoaderOptions, processors?: Processor[]): LoaderFunction
    env(processors?: Processor[]): LoaderFunction
    args(processors?: Processor[]): LoaderFunction
  }

  export type ProcessorCallback = (config: any, cb: Function) => void
  export type ProcessorCallbackDecrypt = (encrypted: any, cb: Function) => void
  export type ProcessorFunctionOptions = (options: ProcessorOptions) => ProcessorCallback
  export type ProcessorFunctionNoOptions = () => ProcessorCallback
  export type ProcessorFunctionDecrypt = (encrypted: string, cb: Function) => ProcessorCallbackDecrypt
  export type ProcessorFunctionJson = (text: string, cb: Function) => void

  export interface ProcessorOptions {
    key?: string
    algorithm?: string
    password?: string
    contentEncoding?: string
    inputEncoding?: string
  }

  export interface ProcessorRegistry {
    mount(options: ProcessorOptions): ProcessorFunctionOptions
    json(): ProcessorFunctionJson
    decrypt(options: ProcessorOptions): ProcessorFunctionDecrypt
    envToProp(): ProcessorCallback
    flatten(): ProcessorFunctionNoOptions
    unflatten(): ProcessorFunctionNoOptions
  }
}

export default Config
export const loaders: Config.LoaderRegistry = Confabulous.loaders
export const processors: Config.ProcessorRegistry = Confabulous.processors