import 'mocha'
import * as assert from 'assert'
import { writeJson, remove } from 'fs-promise'
import System from 'corpjs-system'
import Config, { loaders, processors } from '../src'

const CONFIG_PATH = './test/config.json'

describe('corpjs-config', () => {

  describe('declarations', () => {

    it('should loaders === Config.loaders', () => {
      assert.strictEqual(Config.loaders, loaders)
    })

    it('should processors === Config.processors', () => {
      assert.strictEqual(Config.processors, processors)
    })

  })

  describe('load and watch configs from file', () => {

    let system: System

    beforeEach(async () => {
      if (system) await system.stop()
      system = undefined
    })

    it('should get config', async () => {
      const config = await createConfigFile({ a: 1 })
      system = new System({ exitOnError: false }).add('config', new Config()
        .add(config => loaders.require({ path: CONFIG_PATH })))
      const resources = await system.start()
      assert.deepStrictEqual(resources, { config: config })
    })

    it('should watch config', done => {
      (async () => {
        let config = await createConfigFile({ a: 1 })
        system = new System({ exitOnError: false }).add('config', new Config()
          .add(config => loaders.require({ path: CONFIG_PATH, watch: true })))
          .once('restart', resources => {
            try {
              assert.deepStrictEqual(resources, { config: config })
              done()
            } catch (err) {
              done(err)
            }
          })
        const resources = await system.start()
        assert.deepStrictEqual(resources, { config: config })
        config = await createConfigFile({ a: 2 })
      })()
    })

    it('should stop the whole system after reload error', done => {
      (async () => {
        let config = await createConfigFile({ a: 1 })
        system = new System({ exitOnError: false }).add('config', new Config()
          .add(config => loaders.require({ path: CONFIG_PATH, watch: true })))
          .once('stop', done)
        const resources = await system.start()
        assert.deepStrictEqual(resources, { config: config })
        await remove(CONFIG_PATH)
      })()
    })

  })

})

async function createConfigFile(config): Promise<any> {
  await writeJson(CONFIG_PATH, config)
  return config
}